# Generated by Django 4.0.5 on 2022-06-19 05:03

import uuid

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("gqlauth", "0002_initial"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="captcha",
            name="id",
        ),
        migrations.AlterField(
            model_name="captcha",
            name="uuid",
            field=models.UUIDField(
                default=uuid.uuid4, editable=False, primary_key=True, serialize=False
            ),
        ),
    ]
