# Generated by Django 4.0.6 on 2022-08-02 19:50

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("gqlauth", "0004_captcha_url"),
    ]

    operations = [
        migrations.RenameField(
            model_name="captcha",
            old_name="url",
            new_name="image",
        ),
    ]
