# Generated by Django 4.0.6 on 2022-08-02 19:57

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("gqlauth", "0005_rename_url_captcha_image"),
    ]

    operations = [
        migrations.AlterField(
            model_name="captcha",
            name="image",
            field=models.ImageField(
                editable=False, help_text="url for the captcha image", upload_to="captcha/%Y/%m/%d/"
            ),
        ),
    ]
